//
//  BigLetterView.m
//  TypingTutor
//
//  Created by narita on 2013/12/25.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#import "BigLetterView.h"

@implementation BigLetterView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    
    if (self)
    {
        NSLog(@"initalize view");
        
        self.bgColor = [NSColor yellowColor];
        self.string = @"";
        [self prepareAttributes];
    }
    return self;
}

- (void) prepareAttributes
{
    self.attributes = [NSMutableDictionary dictionary];
    
    self.attributes[NSFontAttributeName]            = [NSFont userFontOfSize:75];
    self.attributes[NSForegroundColorAttributeName] = [NSColor redColor];
    
    NSShadow* theShadow = [[NSShadow alloc] init];
    theShadow.shadowOffset = NSMakeSize(3.0, -3.0);
    self.attributes[NSShadowAttributeName]          = theShadow;
}

@synthesize bgColor = _bgColor;

- (void) setBgColor:(NSColor *)bgColor
{
    _bgColor = bgColor;
    [self setNeedsDisplay:YES];
}

- (NSColor*) bgColor
{
    return _bgColor;
}

@synthesize string = _string;

- (void) setString:(NSString *)string
{
    _string = string;
    
    NSLog(@"setString:%@", string);

    [self setNeedsDisplay:YES];
}

- (NSString*) string
{
    return _string;
}

@synthesize bold = _bold;

- (void) setBold:(BOOL)inBold
{
    NSFont*        theFont = self.attributes[NSFontAttributeName];
    NSFontManager* theFontManager = [NSFontManager sharedFontManager];

    if( inBold )
    {
        theFont = [theFontManager convertFont:theFont
                                  toHaveTrait:NSBoldFontMask];
    }
    else
    {
        theFont = [theFontManager convertFont:theFont
                               toNotHaveTrait:NSBoldFontMask];
    }
    
    self.attributes[NSFontAttributeName]            = theFont;

    [self setNeedsDisplay:YES];
}

- (BOOL) isBold
{
    NSFontManager* theFontManager = [NSFontManager sharedFontManager];

    if( [theFontManager traitsOfFont:self.attributes[NSFontAttributeName]] & NSBoldFontMask )
    {
        return YES;
    }
    else
    {
        return NO;
    }
}

@synthesize italic = _italic;

- (void) setItalic:(BOOL)inItalic
{
    NSFont*        theFont = self.attributes[NSFontAttributeName];
    NSFontManager* theFontManager = [NSFontManager sharedFontManager];
    
    if( inItalic )
    {
        theFont = [theFontManager convertFont:theFont
                                  toHaveTrait:NSItalicFontMask];
    }
    else
    {
        theFont = [theFontManager convertFont:theFont
                               toNotHaveTrait:NSItalicFontMask];
    }
    
    self.attributes[NSFontAttributeName]            = theFont;
    
    [self setNeedsDisplay:YES];
}

- (BOOL) isItalic
{
    NSFontManager* theFontManager = [NSFontManager sharedFontManager];
    
    if( [theFontManager traitsOfFont:self.attributes[NSFontAttributeName]] & NSItalicFontMask )
    {
        return YES;
    }
    else
    {
        return NO;
    }
}


// 描画系

- (void) drawStringCenteredIn:(NSRect) inRect
{
    NSSize  theStringSize = [self.string sizeWithAttributes:self.attributes];
    NSPoint theStringOrigin;
    
    theStringOrigin.x = inRect.origin.x + (inRect.size.width - theStringSize.width) / 2;
    theStringOrigin.y = inRect.origin.y + (inRect.size.height - theStringSize.height) / 2;
    
    [self.string drawAtPoint:theStringOrigin withAttributes:self.attributes];
}

- (void)drawRect:(NSRect)dirtyRect
{
    NSRect theBounds = [self bounds];
    
    [self.bgColor set];
    [NSBezierPath fillRect:theBounds];
    
    [self drawStringCenteredIn:theBounds];
    
    if(
       [[self window] firstResponder] == self
       &&
       [NSGraphicsContext currentContextDrawingToScreen]
       )
    {
        [NSGraphicsContext saveGraphicsState];
        
        NSSetFocusRingStyle(NSFocusRingOnly);
        [NSBezierPath fillRect:theBounds];
        
        [NSGraphicsContext restoreGraphicsState];
    }
}

- (BOOL) isOpaque
{
    return YES;
}

- (IBAction) savePDF:(id)sender
{
    __block NSSavePanel* theSavePanel = [NSSavePanel savePanel];
    
    [theSavePanel setAllowedFileTypes:@[@"pdf"]];
    
    [theSavePanel beginSheetModalForWindow:self.window
                         completionHandler:^(NSInteger inResult)
     {
         if(inResult == NSOKButton)
         {
             NSRect theBounds = self.bounds;
             NSData* theData  = [self dataWithPDFInsideRect:theBounds];
             NSError* theError;
             
             BOOL theSuccessful = [theData writeToURL:[theSavePanel URL]
                                              options:0
                                                error:&theError];
             
             if( !theSuccessful )
             {
                 NSAlert* theAlert = [NSAlert alertWithError:theError];
                 [theAlert runModal];
             }
         }
         
         theSavePanel = nil;
     }];
    
}


// firstResponder系
- (BOOL) acceptsFirstResponder
{
    NSLog(@"accepting");
    return YES;
}

- (BOOL) resignFirstResponder
{
    NSLog(@"resigning");
    
    [self setKeyboardFocusRingNeedsDisplayInRect:[self bounds]];
    
    return YES;
}

- (BOOL) becomeFirstResponder
{
    NSLog(@"becoming");
    
    [self setNeedsDisplay:YES];
    
    return YES;
}

// キー入力系

- (void) keyDown:(NSEvent *)theEvent// メニューにあるショートカットは送られてこない
{
    [self interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
}

// keyUpだけ送られてくるときもある。例えば、前面のWindowがescでクローズされた場合はescがkeyUpされたイベントがクローズされたwindow以外に飛んで来る
//- (void)keyUp:(NSEvent *)theEvent
//{
//    NSLog(@"keyUp:");
//}
//
- (void) insertText:(id)insertString
{
    self.string = insertString;
}

- (void) insertTab:(id)sender
{
    [[self window] selectKeyViewFollowingView:self];
}

- (void) insertBacktab:(id)sender
{
    [[self window] selectKeyViewPrecedingView:self];
}

- (void) deleteBackward:(id)sender
{
    self.string = @"";
}

- (void)writeToPasteboard:(NSPasteboard*)pb
{
    [pb clearContents];
    
    // pdfデータの準備
    NSData* thePDFData  = [self dataWithPDFInsideRect:self.bounds];

    NSPasteboardItem* thePDFItem = [[NSPasteboardItem alloc] init];
    [thePDFItem setData:thePDFData forType:NSPasteboardTypePDF];

    NSPasteboardItem* theTextItem = [[NSPasteboardItem alloc] init];
    [theTextItem setString:self.string forType:NSPasteboardTypeString];

    [pb writeObjects:@[thePDFItem, theTextItem]];
}

- (BOOL)readFromPasteboard:(NSPasteboard*)pb
{
    NSArray* classes = [NSArray arrayWithObject:[NSString class]];
    NSArray* objects = [pb readObjectsForClasses:classes options:nil];
    
    if( [objects count] > 0 )
        {
        NSString* value = [objects objectAtIndex:0];
        
        if([value length] == 1)
            {
            [self setString:value];
            return YES;
            }
        }
    
    return NO;
}

- (IBAction) cut:(id)sender
{
    [self copy:sender];
    [self setString:@""];
}

- (IBAction) copy:(id)sender
{
    NSPasteboard* pb = [NSPasteboard generalPasteboard];
    
    [self writeToPasteboard:pb];
}

- (IBAction) paste:(id)sender
{
    NSPasteboard* pb = [NSPasteboard generalPasteboard];
    
    if( ![self readFromPasteboard:pb] )
        {
        NSBeep();
        }
}

@end
